clc
clear all
close all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Question LLN
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1.

N=1000;
% for the Gaussian law: be careful because the parameters of the matlab
% functions for the gaussian law are the mean and the STANDARD DEVIATION, while
% usually in the literature you will find the mean and the VARIANCE as
% parameters of the gaussian law.
% Always use the help file for the function so that you know what
% parameters you need.
NormSimul=normrnd(3,5^0.5,N,1);

% for the uniform law:
UnifSimul=unifrnd(0,6,N,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2.
% with matlab you can compute the calculations in the compact form:
NormMeans=cumsum(NormSimul,1)./(1:N)';
UnifMeans=cumsum(UnifSimul,1)./(1:N)';

% We plot the empirical estimators of the mean as function of the
% observations.
% For the 2 samples the theoric mean is 3.

x_grid=1:N;
plot(x_grid,NormMeans,x_grid,UnifMeans);
title('Convergence of the empirical mean');
legend('Normal','Uniform');
xlabel('Number of observations');
ylabel('Mean estimator');

% Observe that if we rerun the program, we will obtain different
% "trajectories" for the mean.

% You need a lot of observations to have a good approximation of the
% theoric mean.
% In many applications you have less than 100 observations. You are then 
% obliged to use what you have, but you have to interpret the results carefully! 
% The estimated mean can be far from the real value and as well the real
% confidence interval can be far from the asymptotic confidence interval.
% You will study a method to obtain a confidence interval in case of
% short samples: the bootstrap.



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 3.

UnifVars=zeros(N,1);
NormVars=zeros(N,1);
tic
for i=1:N
   UnifVars(i)=var(UnifSimul(1:i)); %var is unbiased variance function, which adjusts by N-1.
                                    %this can be changed to population
                                    %variance in the options of the var
                                    %function. See help file.
   NormVars(i)=var(NormSimul(1:i));
end
toc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% An alternative (faster) way to compute the cumulative variance
% is to exploit the relation Var[X]=E[X^2]-E[X]^2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic
UnifVars2=cumsum(UnifSimul.^2,1)./(1:N)'-(cumsum(UnifSimul,1)./(1:N)').^2;
NormVars2=cumsum(NormSimul.^2,1)./(1:N)'-(cumsum(NormSimul,1)./(1:N)').^2;
toc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure   % opens a new figure window
plot(x_grid,NormVars,x_grid,UnifVars);
title('Convergence of the empirical variance');
legend('Normal','Uniform');
xlabel('Number of observations');
ylabel('Variance estimation');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Remind:
% variance of a variable U(a,b) = (b-a)^2/12
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure   
plot(x_grid,NormVars2,x_grid,UnifVars2);
title('Convergence of the empirical variance');
legend('Normal','Uniform');
xlabel('Number of observations');
ylabel('Variance estimation');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Question CLT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1.

% Simulation of the gaussian sample:
NormSimuls=normrnd(3,5^0.5,10000,100);
% Calculation of the means:
NormMoys20=mean(NormSimuls(1:20,:),1);
NormMoys50=mean(NormSimuls(1:50,:),1);
NormMoys100=mean(NormSimuls(1:100,:),1);
NormMoys500=mean(NormSimuls(1:500,:),1);
NormMoys1000=mean(NormSimuls(1:1000,:),1);

% Simulation of the sample from the uniform distribution:
UnifSimuls=unifrnd(0,6,1000,100);
% Calculation of the means:
UnifMoys20=mean(UnifSimuls(1:20,:),1);
UnifMoys50=mean(UnifSimuls(1:50,:),1);
UnifMoys100=mean(UnifSimuls(1:100,:),1);
UnifMoys500=mean(UnifSimuls(1:500,:),1);
UnifMoys1000=mean(UnifSimuls(1:1000,:),1);

%Programming Tip:
%You can simplify the above steps by running a loop of the form:
%z=[20 50 100 500 1000];
%for i=1:size(z,2)
%    zNormMoys(i,:)=mean(NormSimuls(1:z(i),:),1);
%    zUnifMoys(i,:)=mean(UnifSimuls(1:z(i),:),1);
%end
%And then run the kernel on the different rows of the zNormMoys and
%zUnifMoys variables. In what follows I will use the original setup to avoid confusion.
    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2.

% We can use the gaussiankernelfunction of the previous TP to estimate the
% densities.
% We look directly at the centralized and normalized laws

PdfNormMoys20=gaussiankernel((NormMoys20-3)*(20^0.5),50);
PdfNormMoys50=gaussiankernel((NormMoys50-3)*(50^0.5),50);
PdfNormMoys100=gaussiankernel((NormMoys100-3)*(100^0.5),50);
PdfNormMoys500=gaussiankernel((NormMoys500-3)*(500^0.5),50);
PdfNormMoys1000=gaussiankernel((NormMoys1000-3)*(1000^0.5),50);

% Plot of the densities adding the theoric distribution :

figure
plot(PdfNormMoys20(1,:),PdfNormMoys20(2,:),'-b',PdfNormMoys50(1,:),...
    PdfNormMoys50(2,:),'-r',PdfNormMoys100(1,:),PdfNormMoys100(2,:),'-y',...
    PdfNormMoys500(1,:),PdfNormMoys500(2,:),'-c',PdfNormMoys1000(1,:),...
    PdfNormMoys1000(2,:),'-g',PdfNormMoys1000(1,:),normpdf(PdfNormMoys1000(1,:),0,5^0.5),'-k');
title('Asymptotic distibutions');
legend('20 Obs.','50 Obs.','100 Obs.','500 Obs.','1000 Obs.','Theo.');
xlabel('Value');
ylabel('Density');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Advanced programming:
% How to produce a graph with a loop that at each iteration 
% adds a curve at the picture
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% figure
% cutoffs=[20 50 100 500 1000];
% colors=['b' 'r' 'y' 'c' 'g'];
% for i=1:5
%     Matrix_name=strcat('PdfNormMoys',num2str(cutoffs(i)));
%     Matrix=eval(Matrix_name);
%     plot(Matrix(1,:),Matrix(2,:),strcat('-',colors(i)));
%    hold on
% end
% plot(PdfNormMoys1000(1,:),normpdf(PdfNormMoys1000(1,:),0,5^0.5),'-k');
% title('Asymptotic distibutions');
% legend('20 Obs.','50 Obs.','100 Obs.','500 Obs.','1000 Obs.','Theo.');
% xlabel('Value');
% ylabel('Density');
% hold off


% Same calculation for the uniform law
PdfUnifMoys20=gaussiankernel((UnifMoys20-3)*(20^0.5),50);
PdfUnifMoys50=gaussiankernel((UnifMoys50-3)*(50^0.5),50);
PdfUnifMoys100=gaussiankernel((UnifMoys100-3)*(100^0.5),50);
PdfUnifMoys500=gaussiankernel((UnifMoys500-3)*(500^0.5),50);
PdfUnifMoys1000=gaussiankernel((UnifMoys1000-3)*(1000^0.5),50);


% Plot of the densities adding the theoric distribution :
figure
plot(PdfUnifMoys20(1,:),PdfUnifMoys20(2,:),'-b',PdfUnifMoys50(1,:),PdfUnifMoys50(2,:),'-r',PdfUnifMoys100(1,:),PdfUnifMoys100(2,:),'-y',PdfUnifMoys500(1,:),PdfUnifMoys500(2,:),'-c',PdfUnifMoys1000(1,:),PdfUnifMoys1000(2,:),'-g',PdfUnifMoys1000(1,:),normpdf(PdfUnifMoys1000(1,:),0,3^0.5),'-k');
title('Asymptotic distribution');
legend('20 Obs.','50 Obs.','100 Obs.','500 Obs.','1000 Obs.','Theo.');
xlabel('Value');
ylabel('Density');

% you can see that it is not straightforward to obtain a good illustration
% of the central limit theorem - you need a bit of luck, don't hesitate to
% redo more times these simulations.
% If everything was perfect you should see that the more observations you
% use for calculating the sample mean, the more the asymptotic distribution
% is similar to the theoric one.
% In all cases, this exercise shows that you should be careful with the
% confidence ntervals based on the asymptotic distribution of the
% estimators.
% Also, you may wish to try simulating 10 or 20 thousand datapoints to see
% if things change...

addpath(Users/Bakalli/Box Sync/Unige/Teaching/Financial Econometrics/2016-2017/TP3/TP3_solution)